/* $NetBSD$ */

/*-
 * Copyright (c)2001 Citrus Project,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Citrus: xpg4dl/FreeBSD/lib/libc/stdio/_fpaux.c,v 1.1 2001/09/28 21:59:17 yamt Exp $
 */

#include <sys/types.h>

#include <stdlib.h>
#include "_fpaux.h"

void *
_fpaux_lookup(struct fpaux_list *head, void *fp)
{
	struct fpaux *aux;

	TAILQ_FOREACH(aux, head, fpa_list) {
		if (aux->fpa_fp == fp)
			return aux+1;
	}

	return 0;
}

void *
_fpaux_alloc(struct fpaux_list *head, void *fp, size_t size)
{
	struct fpaux *aux;

	size += sizeof(*aux);
 	aux = malloc(size);
	if (!aux)
		return 0;
	memset(aux, 0, size);
	aux->fpa_fp = fp;
	TAILQ_INSERT_HEAD(head, aux, fpa_list);

	return aux+1;
}

void
_fpaux_free(struct fpaux_list *head, void *fp)
{
	struct fpaux *aux;

	aux = _fpaux_lookup(head, fp);
	if (aux) {
		aux--;
		TAILQ_REMOVE(head, aux, fpa_list);
		free(aux);
	}
}
