/* $NetBSD$ */

/*-
 * Copyright (c)2001 Citrus Project,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Citrus: xpg4dl/FreeBSD/lib/libc/stdio/wcio.c,v 1.1 2001/09/28 21:59:17 yamt Exp $
 */

#include <sys/types.h>
#include <stdio.h>
#include <wchar.h>
#include "_fpaux.h"
#include "reentrant.h"
#include "wcio.h"

/* FPWCIO_HASHSIZE must be a power of 2 */
#define FPWCIO_HASHSIZE 16
#define FPWCIO_HASHMASK (FPWCIO_HASHSIZE-1)
#define FPWCIO_HASH(p) &fp_wcio[(u_int)(((((u_long)p)>>8)^(u_long)p)&FPWCIO_HASHMASK)]
static struct fpaux_list fp_wcio[FPWCIO_HASHSIZE];
#ifdef _REENT
/*
 * we have single lock for fp_wcio since
 * we don't have chances to initialize rwlock_t array.
 */
static rwlock_t fp_wcio_lock = RWLOCK_INITIALIZER;
#endif

struct wchar_io_data *
_wcio_get(FILE *fp)
{
	struct wchar_io_data *wcio;
	struct fpaux_list *head = FPWCIO_HASH(fp);

	rwlock_wrlock(&fp_wcio_lock);
	wcio = _fpaux_lookup(head, fp);
	if (!wcio)
		wcio = _fpaux_alloc(head, fp, sizeof(struct wchar_io_data));
	rwlock_unlock(&fp_wcio_lock);

	return wcio;
}

void
_wcio_free(FILE *fp)
{
	struct fpaux_list *head = FPWCIO_HASH(fp);

	rwlock_wrlock(&fp_wcio_lock);
	_fpaux_free(head, fp);
	rwlock_unlock(&fp_wcio_lock);
}

#if 0 /* unused */
void
_wcio_clear_ungetwc(FILE *fp)
{
	struct wchar_io_data *wcio;
	struct fpaux_list *head = FPWCIO_HASH(fp);

	rwlock_rdlock(&fp_wcio_lock);
	wcio = _fpaux_lookup(head, fp);
	rwlock_unlock(&fp_wcio_lock);
	if (wcio) {
		wcio->wcio_ungetwc_inbuf = 0;
	}	
}
#endif
